#include "RayTracer.h"

int windowX = 640;
int windowY = 480;

std::vector<Object*> objects;

void cleanup() {
	for(unsigned int i = 0; i < objects.size(); ++i){
		if(objects[i]){
			delete objects[i];
		}
	}
}

/*
** TODO: Function for testing intersection against all the objects in the scene
**
** If an object is hit then the IntersectionInfo object should contain
** the information about the intersection. Returns true if any object is hit, 
** false otherwise
**
*/
bool CheckIntersection(const Ray &ray, IntersectInfo &info) {
	return true;	
}

/*
** TODO: Recursive ray-casting function
**
** This function is called for each pixel, and each time a ray is reflected/used 
** for shadow testing. The Payload object can be used to record information about
** the ray trace such as the current color and the number of bounces performed.
** This function should return either the time of intersection with an object
** or minus one to indicate no intersection.
*/
float CastRay(Ray &ray, Payload &payload) {
	
  IntersectInfo info;
	
  if (CheckIntersection(ray,info)) {
    /* TODO: Set payload color based on object materials, not direction */
		payload.color = ray.direction;
    return info.time;
	}
  
	return -1.0f;
}

/*
** Render Function
**
** This is the main render function, it draws pixels onto the display
** using GL_POINTS. It is called every time an update is required.
**
** This function transforms each pixel into the space of the virtual
** scene and casts a ray from the camera in that direction using CastRay
*/
void Render() {

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glm::mat4 viewMatrix = glm::lookAt(glm::vec3(-10.0f,10.0f,10.0f), glm::vec3(0.0f,0.0f,0.0f), glm::vec3(0.0f,1.0f,0.0f));
	glm::mat4 projMatrix = glm::perspective(45.0f, (float)windowX / (float)windowY, 1.0f, 10000.0f);

	glBegin(GL_POINTS);
  
	for(int x = 0; x < windowX; ++x)
  for(int y = 0; y < windowY; ++y){

      float pixelX =  2*((x+0.5f)/windowX)-1;
      float pixelY = -2*((y+0.5f)/windowY)+1;
      
      glm::vec4 worldNear = glm::inverse(viewMatrix) * glm::inverse(projMatrix) * glm::vec4(pixelX, pixelY, -1, 1);
      glm::vec4 worldFar  = glm::inverse(viewMatrix) * glm::inverse(projMatrix) * glm::vec4(pixelX, pixelY,  1, 1);
      
      glm::vec3 worldNearPos = glm::vec3(worldNear.x, worldNear.y, worldNear.z) / worldNear.w;
      glm::vec3 worldFarPos  = glm::vec3(worldFar.x, worldFar.y, worldFar.z) / worldFar.w;
      
			Payload payload;
			Ray ray(worldNearPos, glm::normalize(glm::vec3(worldFarPos - worldNearPos)));	

			if(CastRay(ray,payload) > 0.0f){
        glColor3f(payload.color.x,payload.color.y,payload.color.z);
			} else {
        glColor3f(1,0,0);
      }

			glVertex3f(pixelX,pixelY,0.0f);
	}
  
	glEnd();
	glFlush();
}

int main(int argc, char **argv) {

  
	glutInit(&argc, argv);
	glutInitWindowSize(windowX, windowY);	
	glutCreateWindow("RayTracer");	
	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);	
	glutDisplayFunc(Render);
  
  /* TODO: Add Objects to scene */
  
	atexit(cleanup);

	glutMainLoop();
}
